<?php

// Exit if accessed directly
if ( !defined( 'ABSPATH' ) ) exit;

// Check if class already exists

if (!class_exists("NNRobots_Popular_Posts_DB")) :

/**
 * NNRobots_Popular_Posts_DB class.
 */
class NNRobots_Popular_Posts_DB {

	/**
	 * views_table_name
	 *
	 * (default value: 'nnrobots_popular_posts_views')
	 *
	 * @var string
	 * @access public
	 * @static
	 */
	public static $views_table_name = 'nnrobots_popular_posts_views';

	/**
	 * comments_table_name
	 *
	 * (default value: 'nnrobots_popular_posts_comments')
	 *
	 * @var string
	 * @access public
	 * @static
	 */
	public static $comments_table_name = 'nnrobots_popular_posts_comments';

	/**
	 * Adds 'nnrobots_popular_post' table to the db
	 *
	 * @access public
	 * @static
	 * @return void
	 */
	public static function createTable() {
		global $wpdb;

		$wpdb->query(
			"CREATE TABLE IF NOT EXISTS `" . self::getCommentsTableName() . "` (
				`id` INT AUTO_INCREMENT PRIMARY KEY,
				`post_id` INT NOT NULL,
				`comment_id` INT NULL UNIQUE,
				`date` DATE NOT NULL
			) ENGINE InnoDB DEFAULT CHARACTER SET utf8 COLLATE utf8_unicode_ci AUTO_INCREMENT=1;"
		);

		$wpdb->query(
			"CREATE TABLE IF NOT EXISTS `" . self::getViewsTableName() . "` (
				`id` INT AUTO_INCREMENT PRIMARY KEY,
				`post_id` INT NOT NULL,
				`views` INT NULL,
				`date` DATE NOT NULL
			) ENGINE InnoDB DEFAULT CHARACTER SET utf8 COLLATE utf8_unicode_ci AUTO_INCREMENT=1;"
		);
	}

	/**
	 * Adds comment to DB
	 *
	 * @access public
	 * @static
	 * @param mixed $post_id
	 * @param mixed $comment_id
	 * @param mixed $date (default: null)
	 * @return void
	 */
	public static function addComment($post_id, $comment_id, $date = null) {

		if (!isset($post_id)) {
			return;
		}

		global $wpdb;

		if (!isset($date) || empty($date)) {
			$date = date("Y-m-d");
		}

        $wpdb->query(
        	$wpdb->prepare(
        		"INSERT IGNORE INTO `" . self::getCommentsTableName() . "` (
        			`id`,
        			`post_id`,
        			`comment_id`,
        			`date`)
        			VALUES (NULL, %d, %d, %s
        		);",
        		$post_id,
        		$comment_id,
        		$date
        	)
        );

	}

	/**
	 * Gets comments from the DB
	 *
	 * @access public
	 * @static
	 * @param mixed $posts (default: null)
	 * @param mixed $start_date (default: null)
	 * @param mixed $end_date (default: null)
	 * @param mixed $comment_id (default: null)
	 * @return void
	 */
	static function getComments($posts = null, $start_date = null, $end_date = null, $comment_id = null) {

		global $wpdb;

		// Get all comments for all post types

		if (!isset($posts) && !isset($start_date) && !isset($end_date) && !isset($comment_id)) {
			$comments = $wpdb->get_results("
				SELECT *
				FROM " . self::getTableName() . "
				WHERE comment_id IS NOT NULL;
			");

			return $comments;
		}

		// Get all comments for specfic posts

		if (isset($posts) && !isset($start_date) && isset($end_date)) {

			if (!is_array($posts)) {
				return null;
			}

			$query = "
				SELECT *
				FROM " . self::getCommentsTableName() . "
				WHERE ";

			foreach ($posts as $post) {
				$query .= "post_id = " . $post->ID . " AND ";
			}

			$query .= "comment_id IS NOT NULL;";

			$comments = $wpdb->get_results($query);

			return $comments;
		}

		// Get comments with date range

		if (isset($posts) && isset($start_date) && isset($end_date)) {

			if (!is_array($posts)) {
				return null;
			}

			$query = "
				SELECT *
				FROM " . self::getCommentsTableName() . "
				WHERE (`date` BETWEEN '" . $start_date . "' AND '" . $end_date . "') AND ";

			foreach ($posts as $post) {
				$query .= " post_id = " . $post->ID . " AND ";
			}

			$query .= "comment_id IS NOT NULL;";

			$comments = $wpdb->get_results($query);

			return $comments;
		}

		// Get single comment

		if (isset($comment_id)) {
			$comments = $wpdb->get_results("
				SELECT *
				FROM " . self::getCommentsTableName() . "
				WHERE comment_id = " . $comment_id . ";
			");

			return $comments;
		}

		return null;
	}

	/**
	 * Delete Comment from DB
	 *
	 * @access public
	 * @static
	 * @param mixed $comment_id
	 * @return void
	 */
	public static function deleteComment($comment_id) {

		if (!isset($comment_id)) {
			return;
		}

		global $wpdb;

		$wpdb->query(
        	$wpdb->prepare(
        		"DELETE FROM `" . self::getCommentsTableName() . "`
        		WHERE `comment_id` = %d;",
        		$comment_id
        	)
        );
	}

	/**
	 * Adds a view to the DB
	 *
	 * @access public
	 * @static
	 * @param mixed $post_id
	 * @return void
	 */
	public static function addView($post_id) {

		if (!isset($post_id)) {
			return;
		}

		// Return false if crawler

		if (self::isBot()) {
			return false;
		}

		global $wpdb;
		$date = date("Y-m-d");

		// Check if entry already exsits

		$query = $wpdb->prepare('SELECT * FROM ' . self::getViewsTableName() . ' WHERE `date` = %s AND `post_id` = %d', $date, $post_id);
		$popular_posts_views = $wpdb->query($query);

		// Entry aleady exists, just add 1

		if (isset($popular_posts_views) && $popular_posts_views != 0) {

			$query = $wpdb->prepare('UPDATE ' . self::getViewsTableName() . ' SET
				`views` = `views` + 1
				WHERE `date` = %s AND `post_id` = %d', $date, $post_id);

			$result = $wpdb->query($query);

		}

		// Entry does not exist, create a new one and set views to 1

		else {
			$query = $wpdb->prepare('INSERT INTO ' . self::getViewsTableName() . ' (
				`id`,
				`post_id`,
				`views`,
				`date`
				) VALUES (NULL, %d, 1, %s)',
				$post_id,
				$date
			);

			$result = $wpdb->query($query);
		}
	}

	/**
	 * Gets views for certain posts
	 *
	 * @access public
	 * @static
	 * @param mixed $posts (default: null)
	 * @param mixed $start_date (default: null)
	 * @param mixed $end_date (default: null)
	 * @return void
	 */
	static function getViews($posts = null, $start_date = null, $end_date = null) {

		global $wpdb;

		// Get all views for all post types

		if (!isset($posts) && !isset($start_date) && !isset($end_date)) {

			$views = $wpdb->get_results("
				SELECT *
				FROM " . self::getViewsTableName());

			return $views;
		}

		// Get all views for specfic posts

		if (isset($posts) && !isset($start_date) && isset($end_date)) {

			if (!is_array($posts)) {
				return null;
			}

			$query = "
				SELECT *
				FROM " . self::getViewsTableName() . "
				WHERE ";

			foreach ($posts as $post) {
				$query .= "post_id = " . $post->ID . " AND ";
			}

			$query = rtrim($query, ' AND ');

			$views = $wpdb->get_results($query);

			return $views;
		}

		// Get all views for specfic posts within a date range

		if (isset($posts) && isset($start_date) && isset($end_date)) {

			if (!is_array($posts)) {
				return null;
			}

			$query = "
				SELECT *
				FROM " . self::getViewsTableName() . "
				WHERE (`date` BETWEEN '" . $start_date . "' AND '" . $end_date . "') AND ";

			foreach ($posts as $post) {
				$query .= " post_id = " . $post->ID . " AND ";
			}

			$query = rtrim($query, ' AND ');

			$views = $wpdb->get_results($query);

			return $views;
		}

		return null;
	}

	/**
	 * Delete a view from the DB
	 *
	 * @access public
	 * @static
	 * @param mixed $id
	 * @return void
	 */
	public static function deleteView($id) {

		if (!isset($id)) {
			return;
		}

		global $wpdb;

		$wpdb->query(
        	$wpdb->prepare(
        		"DELETE FROM `" . self::getViewsTableName() . "`
        		WHERE `id` = %d;",
        		$id
        	)
        );
	}

	/**
	 * Check for BOTs visiting site
	 *
	 * @access public
	 * @static
	 * @return void
	 */
	static function isBot() {

		if (isset($_SERVER['HTTP_USER_AGENT']) && preg_match('/bot|spider|crawler|curl|^$/i', $_SERVER['HTTP_USER_AGENT'])) {
			return true;
		} else {
			return false;
		}

	}

	/**
	 * Returns the comments table name
	 *
	 * @access public
	 * @static
	 * @return void
	 */
	static function getCommentsTableName() {

		global $wpdb;

	    return $wpdb->prefix . self::$comments_table_name;
	}

	/**
	 * Returns the views table name
	 *
	 * @access public
	 * @static
	 * @return void
	 */
	static function getViewsTableName() {

		global $wpdb;

	    return $wpdb->prefix . self::$views_table_name;
	}
}

endif;