<?php
/*
Plugin Name: Popular Posts Pro by 99 Robots
plugin URI: http://99robots.com/products/popular-posts-pro
Description: Popular Posts Pro is the most extensive plugin on the market for displaying your blog's more popular posts.
version: 2.2.0
Author: 99 Robots
Author URI: http://99robots.com
License: GPL2
*/

// Exit if accessed directly
if ( !defined( 'ABSPATH' ) ) exit;

/**
 * Global Definitions
 */

/* Item Name */

if (!defined('NNROBOTS_POPULAR_POSTS_ITEM_NAME'))
    define('NNROBOTS_POPULAR_POSTS_ITEM_NAME', 'Popular Posts Pro');

/* Store URL */

if (!defined('NNROBOTS_POPULAR_POSTS_STORE_URL'))
    define('NNROBOTS_POPULAR_POSTS_STORE_URL', 'https://99robots.com');

/* Minimum PHP Version */

if (!defined('NNROBOTS_POPULAR_POSTS_MIN_PHP_VERSION'))
    define('NNROBOTS_POPULAR_POSTS_MIN_PHP_VERSION', '5.4.0');

/* Plugin Name */

if (!defined('NNROBOTS_POPULAR_POSTS_PLUGIN_NAME'))
    define('NNROBOTS_POPULAR_POSTS_PLUGIN_NAME', trim(dirname(plugin_basename(__FILE__)), '/'));

/* Plugin directory */

if (!defined('NNROBOTS_POPULAR_POSTS_PLUGIN_DIR'))
    define('NNROBOTS_POPULAR_POSTS_PLUGIN_DIR', plugin_dir_path(__FILE__) );

/* Plugin url */

if (!defined('NNROBOTS_POPULAR_POSTS_PLUGIN_URL'))
    define('NNROBOTS_POPULAR_POSTS_PLUGIN_URL', plugins_url() . '/' . NNROBOTS_POPULAR_POSTS_PLUGIN_NAME);

/* Plugin verison */

if (!defined('NNROBOTS_POPULAR_POSTS_VERSION_NUM'))
    define('NNROBOTS_POPULAR_POSTS_VERSION_NUM', '2.2.0');

// Required Files

require_once('core/model/db.php');
require_once('core/functions.php');
require_once('admin/functions.php');
require_once('widget.php');
require_once('shortcode.php');

// Hooks / Filters

add_shortcode('nnr_popular_posts', 							'nnr_popular_posts_shortcode');
register_activation_hook( __FILE__, 						array('NNRobots_Popular_Posts', 'register_activation'));
add_action( 'init', 										array('NNRobots_Popular_Posts', 'load_textdomain'));
add_action( 'admin_menu', 									array('NNRobots_Popular_Posts', 'menu_page'));
$plugin = plugin_basename(__FILE__);
add_filter("plugin_action_links_$plugin", 					array('NNRobots_Popular_Posts', 'settings_link'));

// Software Licensing Updates

add_action('wp_ajax_nnr_pp_license_action', array('NNRobots_Popular_Posts', 'license_action'));

if ( !class_exists('EDD_SL_Plugin_Updater') ) {
	include_once( dirname( __FILE__ ) . '/include/edd-software-licensing-updates/EDD_SL_Plugin_Updater.php' );
}

$edd_updater = new EDD_SL_Plugin_Updater( NNROBOTS_POPULAR_POSTS_STORE_URL , __FILE__, array(
		'version' 	=> NNROBOTS_POPULAR_POSTS_VERSION_NUM,
		'license' 	=> NNRobots_Popular_Posts::get_license_key(),
		'item_name' => NNROBOTS_POPULAR_POSTS_ITEM_NAME,
		'author' 	=> '99 Robots',
	)
);

/**
 *  NNRobots_Popular_Posts main class
 *
 * @since 1.0.0
 * @using Wordpress 3.8
 */

class NNRobots_Popular_Posts {

	/**
	 * text_domain
	 *
	 * (default value: 'nnrobots-popular-posts')
	 *
	 * @var string
	 * @access private
	 * @static
	 */
	static $text_domain = 'nnrobots-popular-posts';

	/**
	 * prefix
	 *
	 * (default value: 'nnrobots_popular_posts_')
	 *
	 * @var string
	 * @access private
	 * @static
	 */
	static $prefix = 'nnrobots_popular_posts_';

	/**
	 * prefix_dash
	 *
	 * (default value: 'nnrobots_popular_posts_')
	 *
	 * @var string
	 * @access private
	 * @static
	 */
	static $prefix_dash = 'nnr-pp-';

	/**
	 * settings_page
	 *
	 * (default value: 'nnrobots-popular-posts-admin-menu-settings')
	 *
	 * @var string
	 * @access private
	 * @static
	 */
	private static $settings_page = 'nnrobots-popular-posts-admin-menu-settings';

	/**
	 * tabs_settings_page
	 *
	 * (default value: 'nnrobots-popular-posts-admin-menu-tab-settings')
	 *
	 * @var string
	 * @access private
	 * @static
	 */
	private static $tabs_settings_page = 'nnrobots-popular-posts-admin-menu-tab-settings';

	/**
	 * usage_page
	 *
	 * (default value: 'nnrobots-popular-posts-admin-menu-usage')
	 *
	 * @var string
	 * @access private
	 * @static
	 */
	private static $usage_page = 'nnrobots-popular-posts-admin-menu-usage';

	/**
	 * default
	 *
	 * @var mixed
	 * @access private
	 * @static
	 */
	static $default = array(
		'exclude_views'		=> 'users'
	);

	/**
	 * Load the text domain
	 *
	 * @since 1.0.0
	 */
	static function load_textdomain() {
		load_plugin_textdomain(self::$text_domain, false, NNROBOTS_POPULAR_POSTS_PLUGIN_DIR . '/languages');
	}

	/**
	 * Hooks to 'register_activation_hook'
	 *
	 * @since 1.0.0
	 */
	static function register_activation() {

		/* Check if multisite, if so then save as site option */

		if (function_exists("is_multisite") && is_multisite()) {
			add_site_option(self::$prefix . 'version', NNROBOTS_POPULAR_POSTS_VERSION_NUM);

			// Create the table and populate it with all existing approved comments

			global $wpdb;

			$blog_ids = $wpdb->get_col( "SELECT blog_id FROM $wpdb->blogs" );
		    $original_blog_id = get_current_blog_id();

		    foreach ( $blog_ids as $blog_id ) {
		        switch_to_blog( $blog_id );

				NNRobots_Popular_Posts_DB::createTable();

				$posts = get_posts(array(
					'posts_per_page' 	=> 500,
					'post_status' 		=> 'publish',
				));

				foreach ($posts as $post) {

					$comments = get_approved_comments($post->ID);

					foreach ($comments as $comment) {

						NNRobots_Popular_Posts_DB::addComment(
							$post->ID,
							$comment->comment_ID,
							date("Y-m-d H:i:s", strtotime($comment->comment_date))
						);

					}
				}
		    }

		    restore_current_blog();

		} else {
			add_option(self::$prefix . 'version', NNROBOTS_POPULAR_POSTS_VERSION_NUM);

			// Create the table and populate it with all existing approved comments

			NNRobots_Popular_Posts_DB::createTable();

			$posts = get_posts(array(
				'posts_per_page' 	=> 500,
				'post_status' 		=> 'publish',
			));

			foreach ($posts as $post) {

				$comments = get_approved_comments($post->ID);

				foreach ($comments as $comment) {

					NNRobots_Popular_Posts_DB::addComment(
						$post->ID,
						$comment->comment_ID,
						date("Y-m-d H:i:s", strtotime($comment->comment_date))
					);

				}
			}
		}
	}

	/**
	 * Hooks to 'plugin_action_links_' filter
	 *
	 * @since 1.0.0
	 */
	static function settings_link($links) {
		$settings_link = '<a href="options-general.php?page=' . self::$settings_page . '">Settings</a>';
		array_unshift($links, $settings_link);

		$widget_link = '<a href="widgets.php">Widget</a>';
		array_unshift($links, $widget_link);
		return $links;
	}

	/**
	 * Perform all the actions with a license
	 *
	 * @access public
	 * @static
	 * @return void
	 */
	static function license_action() {

		if ( !isset($_POST['license_key']) ) {
			echo '';
			die();
		}

		// data to send in our API request

		$api_params = array(
			'edd_action'	=> $_POST['license_action'],
			'license' 		=> $_POST['license_key'],
			'item_name' 	=> urlencode( NNROBOTS_POPULAR_POSTS_ITEM_NAME ), // the name of our product in EDD
			'url'       	=> is_multisite() ? network_home_url() : home_url()
		);

		// Call the custom API.

		$response = wp_remote_post( NNROBOTS_POPULAR_POSTS_STORE_URL, array( 'timeout' => 15, 'sslverify' => false, 'body' => $api_params ) );

		// make sure the response came back okay

		if ( is_wp_error( $response ) ) {
			echo $response;
			die();
		}

		$license_data = json_decode( wp_remote_retrieve_body( $response ) );

		if ( $_POST['license_action'] == 'activate_license' ) {
			self::update_license_key($_POST['license_key']);
		} else if ( $_POST['license_action'] == 'deactivate_license' ) {
			self::update_license_key('');
		}

		// decode the license data

		echo $license_data->license;

		die();

	}

	/**
	 * Check if license key is active
	 *
	 * @access public
	 * @static
	 * @param mixed $license_key
	 * @return void
	 */
	static function is_license_active( $license_key ) {

		if ( !isset( $license_key ) ) {
			return false;
		}

		// data to send in our API request

		$api_params = array(
			'edd_action'	=> 'check_license',
			'license' 		=> $license_key,
			'item_name' 	=> urlencode( NNROBOTS_POPULAR_POSTS_ITEM_NAME ), // the name of our product in EDD
			'url'       	=> is_multisite() ? network_home_url() : home_url()
		);

		// Call the custom API.

		$response = wp_remote_post( NNROBOTS_POPULAR_POSTS_STORE_URL, array( 'timeout' => 15, 'sslverify' => false, 'body' => $api_params ) );

		// make sure the response came back okay

		if ( is_wp_error( $response ) ) {
			return false;
		}

		$license_data = json_decode( wp_remote_retrieve_body( $response ) );

		// decode the license data

		if ( isset($license_data->license) && $license_data->license == 'valid' ) {
			return true;
		} else {
			return false;
		}

	}

	/**
	 * Get the license key
	 *
	 * @access public
	 * @static
	 * @return void
	 */
	static function get_license_key() {

		// Get the license key from based on WordPress install

		if ( function_exists('is_multisite') && is_multisite() ) {
			$license_key = get_site_option(self::$prefix . 'license_key');
		} else {
			$license_key = get_option(self::$prefix . 'license_key');
		}

		// Check if license key is set

		if ( $license_key === false ) {
			$license_key = '';
		}

		return $license_key;
	}

	/**
	 * Update the license key
	 *
	 * @access public
	 * @static
	 * @param mixed $license_key
	 * @return void
	 */
	static function update_license_key($license_key) {

		if ( isset( $license_key ) ) {

			// Get the license key from based on WordPress install

			if ( function_exists('is_multisite') && is_multisite() ) {

				update_site_option(self::$prefix . 'license_key', $license_key);

			} else {

				update_option(self::$prefix . 'license_key', $license_key);

			}
		}
	}

	/**
	 * Hooks to 'admin_menu'
	 *
	 * @since 1.0.0
	 */
	static function menu_page() {

	    // Cast the first sub menu to the top menu

	    $settings_page_load = add_submenu_page(
	    	'options-general.php', 										// parent slug
	    	__('Popular Posts Pro', self::$text_domain), 				// Page title
	    	__('Popular Posts Pro', self::$text_domain), 				// Menu name
	    	'manage_options', 											// Capabilities
	    	self::$settings_page, 										// slug
	    	array('NNRobots_Popular_Posts', 'admin_settings')			// Callback function
	    );
	    add_action("admin_print_scripts-$settings_page_load", array('NNRobots_Popular_Posts', 'include_admin_scripts'));
	}

	/**
	 * Hooks to 'admin_print_scripts-$page'
	 *
	 * @since 1.0.0
	 */
	static function include_admin_scripts() {

		// Styles

		wp_enqueue_style(self::$prefix . 'settings_css', 			NNROBOTS_POPULAR_POSTS_PLUGIN_URL . '/admin/css/settings.css');
		wp_enqueue_style(self::$prefix . 'bootstrap_css', 			NNROBOTS_POPULAR_POSTS_PLUGIN_URL . '/admin/css/nnr-bootstrap.min.css');
		wp_enqueue_style(self::$prefix_dash . 'fontawesome_css',	NNROBOTS_POPULAR_POSTS_PLUGIN_URL . '/include/fontawesome/css/font-awesome.min.css');

		// Scripts

		wp_enqueue_script(self::$prefix . 'bootstrap_js', 			NNROBOTS_POPULAR_POSTS_PLUGIN_URL . '/admin/js/bootstrap.min.js', array('jquery'));
		wp_enqueue_script(self::$prefix . 'settings_js', 			NNROBOTS_POPULAR_POSTS_PLUGIN_URL . '/admin/js/settings.js', array('jquery'));
		wp_localize_script(self::$prefix . 'settings_js', 			'nnr_pp_data', array(
			'prefix_dash'		=> self::$prefix_dash,
			'activate_text'		=> __('Activate', self::$text_domain),
			'deactivate_text'	=> __('Deactivate', self::$text_domain),
			'no_license_key'		=> __('Please add or activate your License Key to get automatic updates.  Without a valid license key you will not receive regular updates. You can find your license key', self::$text_domain) . ' <a href="https://99robots.com/dashboard" target="_blank">here</a>',
			'expired'				=> __('EXPIRED: This license key has expired.  Please renew your license key', self::$text_domain) . ' <a href="https://99robots.com/dashboard" target="_blank">here</a>',
			'activation_error'		=> __('INVALID LICENSE KEY: The license key is not valid please try again. You can find your license key', self::$text_domain) . ' <a href="https://99robots.com/dashboard" target="_blank">here</a>.',
			'activation_valid'		=> __('SUCCESS: Your license key is valid.', self::$text_domain),
			'deactivation_valid'	=> __('SUCCESS: This site has been deactivated.', self::$text_domain),
			'deactivation_error'	=> __('DEACTIVATION FAILED: This site could not be deactivated.', self::$text_domain),
		));
	}

	/**
	 * Displays the HTML for the 'nnrobots-popular-posts-admin-menu-settings' admin page
	 *
	 * @since 1.0.0
	 */
	static function admin_settings() {

		global $wp_version;

		$license_key = self::get_license_key();

		$settings = get_option(self::$prefix . 'settings');

		// Default values

		if ($settings === false) {
			$settings = self::$default;
		}

		// Save data nd check nonce

		if (isset($_POST['submit']) && check_admin_referer(self::$prefix . 'admin_settings')) {

			$settings = array(
				'exclude_views'	=> stripcslashes(sanitize_text_field($_POST[self::$prefix_dash . 'exclude-views'])),
			);

			update_option(self::$prefix . 'settings', $settings);
		}

		require('admin/views/settings.php');
	}
}